﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PServiceBus.Gateway.Interfaces;
using Phoenix.RedisConnectors;
using PServiceBus.Core.Runtime;

namespace PServiceBus.Gateway.Redis {

    public class RedisGatewayServer : IGatewayServer {

        private RedisReceiver _server;

        public RedisGatewayServer() { }

        public RedisGatewayServer(string address) {
            var endpoint = EndpointUri.Parse(address);
            _server = new RedisReceiver(RedisConstants.RPCServiceName, endpoint.LocalizedHost, endpoint.Port);
        }

        #region IGatewayServer Members

        public Action<Exception> ErrorReceived { get; set; }

        public void Register<TCommand>(Action<TCommand> command) where TCommand : class {
            _server.Register<TCommand>(command);
        }

        public void Register<TRequest, TResponse>(Func<TRequest, TResponse> command)
            where TRequest : class
            where TResponse : class {
                _server.Register<TRequest, TResponse>(command);
        }

        public void Start() {
            _server.Start();
        }

        public IGatewayServer Init(string address) {
            return new RedisGatewayServer(address);
        }

        #endregion

        #region IDisposable Members

        public void Dispose() {
            if (_server != null)
                _server.Dispose();
        }

        #endregion
    }
}
