﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PServiceBus.Core.Runtime.Gateway;
using Phoenix.ZeroMQConnectors;
using PServiceBus.Core.Interface;
using PServiceBus.Core.Security;
using PServiceBus.Gateway.Interfaces;
using PServiceBus.Gateway.Interfaces.DTO;
using PServiceBus.Gateway.ZeroMQ.DTO;
using PServiceBus.Gateway.Interfaces.Exceptions;
using PServiceBus.Core.Runtime;
using PServiceBus.Gateway.ZeroMQ.Commands;
using PServiceBus.Gateway.Interfaces.Commands;

namespace PServiceBus.Gateway.ZeroMQ {
    public class ZeroMQMessageBus : GatewayMessageBusBase {

        public ZeroMQMessageBus() { }

        static ZeroMQMessageBus() {
            ZeroMQSerializationBootstrapper.Configure();
            ServiceLocator<IBusRouter>.Set(new ZeroMQBusRouter());
            ServiceLocator<IGatewayServer>.Set(new ZeroMQGatewayServer());
            ServiceLocator<IGatewayClient>.Set(new ZeroMQGatewayClient());
            ServiceLocator<IGatewayService>.Set(new ZeroMQGatewayService());
            ServiceLocator<IRouterService>.Set(new ZeroMQRouterService());
        }

        public ZeroMQMessageBus(string address) : base(address) { }

        #region GatewayMessageBusBase Members
        public override IGatewayMessageBus Init(string address) {
            return new ZeroMQMessageBus(address);
        }

        public override void CreateSubscriber(string subscriber) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void ChangeSubscriberDurability(string subscriber, bool durable) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber, durable: durable, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void SubscribeTo(string subscriber, string topicName, string filter, bool needHeader, bool caseSensitive) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber,
            topicName: topicName, filter: filter, needHeader: needHeader, async: false, caseSensitive: caseSensitive, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void UpdateSubscriber(string subscriber, string topicName, string filter, bool needHeader, bool caseSensitive) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber,
            topicName: topicName, filter: filter, needHeader: needHeader, async: false, caseSensitive: caseSensitive, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void UnSubscribeFrom(string subscriber, string topicName) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber, topicName: topicName, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void AddTransport(string subscriber, string transportName, ITransport transport, string topicName) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber, transportName: transportName,
                transport: transport, topicName: topicName, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void DeleteTransport(string subscriber, string transportName) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: subscriber, transportName: transportName, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override SubscriberInfo SelectSubscriber(string name) {
            return Send<ZeroMQESBRequest, ZeroMQSubscriberInfo>(ZeroMQESBRequest.New(name, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override TopicInfo SelectTopic(string name) {
            var response = Send<ZeroMQESBRequest, ZeroMQESBTopicInfo>(ZeroMQESBRequest.New(name, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
            return response != null ? response.Topic : null;
        }

        public override void DeleteSubscriber(string name) {
            Send<BusMessage>(ZeroMQBusMessage.New(subscriberName: name, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void RegisterTopic(TopicInfo topic) {
            Send<BusMessage>(ZeroMQBusMessage.New(topic: topic, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void PublishTopic(TopicInfo topic) {
            Send<BusMessage>(ZeroMQBusMessage.New(topic: topic, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override void DeleteTopic(string name) {
            Send<BusMessage>(ZeroMQBusMessage.New(topicName: name, async: false, userIdentity: Identity, reThrowException: ReThrowException, headers: Headers));
        }

        public override IList<TopicInfo> GetTopics() {
            return Send<ZeroMQESBRequest, ZeroMQTopicInfos>(ZeroMQESBRequest.New(userIdentity: Identity, reThrowException: ReThrowException, headers: Headers)).Topics;
        }

        public override bool IsValidException(Exception ex) {
            var exName = ex.GetType().Name;
            var isZeroMQException = exName.Contains("ZmqSocketException") || exName.Contains("ZmqException");
            if (!isZeroMQException && ex is ZeroMQFaultException) 
                throw ex;
            return isZeroMQException;
        }
        #endregion
    }
}
